@@ -51,6 +51,16 @@
 
 #include "cd-drive.h"
 
+static struct {
+	const char *name;
+	gboolean can_write_cdr;
+	gboolean can_write_cdrw;
+	gboolean can_write_dvdr;
+	gboolean can_write_dvdram;
+} recorder_whitelist[] = {
+	{ "IOMEGA - CDRW9602EXT-B", TRUE, TRUE, FALSE, FALSE },
+};
+
 #if defined(__linux__) || defined(__FreeBSD__)
 
 /* For dvd_plus_rw_utils.cpp */
@@ -78,7 +88,7 @@ add_dvd_plus (CDDrive *cdrom)
 	}
 }
 
-static CDBlankMediaType
+static CDMediaType
 linux_bsd_media_type (const char *device)
 {
 	int fd;
@@ -86,27 +96,36 @@ linux_bsd_media_type (const char *device)
 
 	fd = open (device, O_RDONLY);
 	if (fd < 0) {
-		return CD_BLANK_MEDIA_TYPE_UNKNOWN;
+		return CD_MEDIA_TYPE_ERROR;
 	}
 
-	mmc_profile = get_mmc_profile ((void *)fd);
+	mmc_profile = get_mmc_profile ((void *)&fd);
 	close (fd);
 
 	switch (mmc_profile) {
 	case -1:        /* Couldn't get the data about the media */
-		return CD_BLANK_MEDIA_TYPE_UNKNOWN;
+		return CD_MEDIA_TYPE_UNKNOWN;
+	case 0x8:	/* Commercial CDs and Audio CD	*/
+		return CD_MEDIA_TYPE_CD;
 	case 0x9:	/* CD-R                         */
-		return CD_BLANK_MEDIA_TYPE_CD;
+		return CD_MEDIA_TYPE_CDR;
+	case 0xa:	/* CD-RW			*/
+		return CD_MEDIA_TYPE_CDRW;
+	case 0x10:	/* Commercial DVDs		*/
+		return CD_MEDIA_TYPE_DVD;
 	case 0x11:      /* DVD-R                        */
+		return CD_MEDIA_TYPE_DVDR;
 	case 0x13:      /* DVD-RW Restricted Overwrite  */
 	case 0x14:      /* DVD-RW Sequential            */
-		return CD_BLANK_MEDIA_TYPE_DVDR;
+		return CD_MEDIA_TYPE_DVDRW;
 	case 0x1B:      /* DVD+R                        */
+		return CD_MEDIA_TYPE_DVD_PLUS_R;
 	case 0x1A:      /* DVD+RW                       */
-		return CD_BLANK_MEDIA_TYPE_DVD_PLUS_R;
+		return CD_MEDIA_TYPE_DVD_PLUS_RW;
 	case 0x12:      /* DVD-RAM                      */
+		return CD_MEDIA_TYPE_DVD_RAM;
 	default:
-		return CD_BLANK_MEDIA_TYPE_UNKNOWN;
+		return CD_MEDIA_TYPE_UNKNOWN;
 	}
 }
 
@@ -177,7 +196,9 @@ struct scsi_unit {
 };
 
 struct cdrom_unit {
+	CDProtocolType protocol;
 	char *device;
+	char *display_name;
 	int speed;
 	gboolean can_write_cdr;
 	gboolean can_write_cdrw;
@@ -186,6 +207,32 @@ struct cdrom_unit {
 	gboolean can_read_dvd;
 };
 
+static char *cdrom_get_name (struct cdrom_unit *cdrom, struct scsi_unit *scsi_units, int n_scsi_units);
+
+static void
+add_whitelist (struct cdrom_unit *cdrom_s,
+	       struct scsi_unit *scsi_units, int n_scsi_units)
+{
+	guint i;
+
+	for (i = 0; i < G_N_ELEMENTS (recorder_whitelist); i++) {
+		if (cdrom_s->display_name == NULL) {
+			continue;
+		}
+
+		if (!strcmp (cdrom_s->display_name, recorder_whitelist[i].name)) {
+			cdrom_s->can_write_cdr =
+				recorder_whitelist[i].can_write_cdr;
+			cdrom_s->can_write_cdrw =
+				recorder_whitelist[i].can_write_cdrw;
+			cdrom_s->can_write_dvdr =
+				recorder_whitelist[i].can_write_dvdr;
+			cdrom_s->can_write_dvdram =
+				recorder_whitelist[i].can_write_dvdram;
+		}
+	}
+}
+
 static void
 get_scsi_units (char **device_str, char **devices, struct scsi_unit *scsi_units)
 {
@@ -305,7 +352,7 @@ get_device_max_speed (char *id)
 	const char *argv[20]; /* Shouldn't need more than 20 arguments */
 	char *dev_str, *stdout_data, *speed;
 
-	max_speed = 1;
+	max_speed = -1;
 
 	i = 0;
 	argv[i++] = "cdrecord";
@@ -327,6 +374,12 @@ get_device_max_speed (char *id)
 		if (speed != NULL) {
 			speed += strlen ("Maximum write speed in kB/s:");
 			max_speed = (int)floor (atol (speed) / 176.0 + 0.5);
+		} else {
+			speed = strstr (stdout_data, "Maximum write speed:");
+			if (speed != NULL) {
+				speed += strlen ("Maximum write speed:");
+				max_speed = (int)floor (atol (speed) / 176.0 + 0.5);
+			}
 		}
 		g_free (stdout_data);
 	}
@@ -368,11 +421,12 @@ cdrom_get_name (struct cdrom_unit *cdrom, struct scsi_unit *scsi_units, int n_sc
 		return NULL;
 	}
 	if (i == 2) {
+		g_free (cdrom->device);
 		cdrom->device = g_strdup(devfsname);
 	}
 	stdname[3] = '\0'; devfsname[14] = '\0'; /* just in case */
 	
-	if (stdname[0] == 's') {
+	if (cdrom->protocol == CD_PROTOCOL_SCSI) {
 		get_cd_scsi_id (cdrom->device, &bus, &id, &lun);
 		retval = get_scsi_cd_name (bus, id, lun, cdrom->device, scsi_units,
 			   n_scsi_units);
@@ -400,6 +454,7 @@ cdrom_get_name (struct cdrom_unit *cdrom, struct scsi_unit *scsi_units, int n_sc
 
 static GList *
 add_linux_cd_recorder (GList *cdroms,
+		       gboolean recorder_only,
 		       struct cdrom_unit *cdrom_s,
 		       struct scsi_unit *scsi_units,
 		       int n_scsi_units)
@@ -410,26 +465,35 @@ add_linux_cd_recorder (GList *cdroms,
 	cdrom = g_new0 (CDDrive, 1);
 
 	cdrom->type = CDDRIVE_TYPE_CD_DRIVE;
+	cdrom->display_name = g_strdup (cdrom_s->display_name);
 
-	if (cdrom_s->device[0] == 's') {
-		cdrom->display_name = cdrom_get_name (cdrom_s, scsi_units,
-				n_scsi_units);
+	if (cdrom_s->protocol == CD_PROTOCOL_SCSI) {
+		cdrom->protocol = CD_PROTOCOL_SCSI;
 		if (!get_cd_scsi_id (cdrom_s->device, &bus, &id, &lun)) {
+			g_free (cdrom->display_name);
+			g_free (cdrom);
 			return cdroms;
 		}
 		cdrom->cdrecord_id = g_strdup_printf ("%d,%d,%d",
 				bus, id, lun);
-		cdrom->max_speed_write = get_device_max_speed
-			(cdrom->cdrecord_id);
 	} else {
+		cdrom->protocol = CD_PROTOCOL_IDE;
 		/* kernel >=2.5 can write cd w/o ide-scsi */
-		cdrom->display_name = cdrom_get_name (cdrom_s,
-				scsi_units, n_scsi_units);
 		cdrom->cdrecord_id = g_strdup_printf ("/dev/%s",
 				cdrom_s->device);
+	}
+
+	if (recorder_only) {
 		cdrom->max_speed_write = get_device_max_speed
 			(cdrom->cdrecord_id);
+		if (cdrom->max_speed_write == -1) {
+			cdrom->max_speed_write = cdrom_s->speed;
+		}
+	} else {
+		/* Have a wild guess, the drive should actually correct us */
+		cdrom->max_speed_write = cdrom_s->speed;
 	}
+
 	cdrom->device = g_strdup_printf ("/dev/%s", cdrom_s->device);
 	cdrom->max_speed_read = cdrom_s->speed;
 	if (cdrom_s->can_write_dvdr) {
@@ -448,10 +512,9 @@ add_linux_cd_recorder (GList *cdroms,
 	}
 	if (cdrom_s->can_read_dvd) {
 		cdrom->type |= CDDRIVE_TYPE_DVD_DRIVE;
+		add_dvd_plus (cdrom);
 	}
 
-	add_dvd_plus (cdrom);
-
 	return g_list_append (cdroms, cdrom);
 }
 
@@ -464,7 +527,7 @@ add_linux_cd_drive (GList *cdroms, struct cdrom_unit *cdrom_s,
 	cdrom = g_new0 (CDDrive, 1);
 	cdrom->type = CDDRIVE_TYPE_CD_DRIVE;
 	cdrom->cdrecord_id = NULL;
-	cdrom->display_name = cdrom_get_name (cdrom_s, scsi_units, n_scsi_units);
+	cdrom->display_name = g_strdup (cdrom_s->display_name);
 	cdrom->device = g_strdup_printf ("/dev/%s", cdrom_s->device);
 	cdrom->max_speed_write = 0; /* Can't write */
 	cdrom->max_speed_read = cdrom_s->speed;
@@ -537,6 +600,8 @@ linux_scan (gboolean recorder_only)
 			*t = 0;
 		}
 		cdroms[j].device = get_cd_device_file (p);
+		/* Assume its an IDE device for now */
+		cdroms[j].protocol = CD_PROTOCOL_IDE;
 		if (t != NULL) {
 			p = t + 1;
 		}
@@ -549,6 +614,7 @@ linux_scan (gboolean recorder_only)
 	n_scsi_units = 0;
 	for (i = 0; i < n_cdroms; i++) {
 		if (cdroms[i].device[0] == 's') {
+			cdroms[i].protocol = CD_PROTOCOL_SCSI;
 			n_scsi_units++;
 		}
 	}
@@ -660,34 +726,48 @@ linux_scan (gboolean recorder_only)
 		maj = min = 0;
 	}
 	fclose(file);
-	
+
 	cdroms_list = NULL;
 	for (i = n_cdroms - 1, j = 0; i >= 0; i--, j++) {
+		if (have_devfs) {
+			char *s;
+			s = g_strdup_printf("%s cdroms/cdrom%d",
+					cdroms[i].device,  j);
+			g_free (cdroms[i].device);
+			cdroms[i].device = s;
+		}
+		cdroms[i].display_name = cdrom_get_name (&cdroms[i],
+				scsi_units, n_scsi_units);
+		add_whitelist (&cdroms[i], scsi_units, n_scsi_units);
+
 		if ((cdroms[i].can_write_cdr ||
 		    cdroms[i].can_write_cdrw ||
 		    cdroms[i].can_write_dvdr ||
 		    cdroms[i].can_write_dvdram) &&
-			(cdroms[i].device[0] == 's' ||
+			(cdroms[i].protocol == CD_PROTOCOL_SCSI ||
 			(maj > 2) || (maj == 2 && min >= 5))) {
-			if (have_devfs) {
-				cdroms[i].device = g_strdup_printf("%s cdroms/cdrom%d",
-						cdroms[i].device,  j);
-			}
 			cdroms_list = add_linux_cd_recorder (cdroms_list,
-					&cdroms[i], scsi_units, n_scsi_units);
+					recorder_only, &cdroms[i],
+					scsi_units, n_scsi_units);
 		} else if (!recorder_only) {
-			if (have_devfs) {
-				cdroms[i].device = g_strdup_printf("%s cdroms/cdrom%d",
-						cdroms[i].device,  j);
-			}
 			cdroms_list = add_linux_cd_drive (cdroms_list,
 					&cdroms[i], scsi_units, n_scsi_units);
 		}
 	}
 
-	g_free (scsi_units);
+	for (i = n_cdroms - 1; i >= 0; i--) {
+		g_free (cdroms[i].display_name);
+		g_free (cdroms[i].device);
+	}
 	g_free (cdroms);
 
+	for (i = n_scsi_units - 1; i >= 0; i--) {
+		g_free (scsi_units[i].vendor);
+		g_free (scsi_units[i].model);
+		g_free (scsi_units[i].rev);
+	}
+	g_free (scsi_units);
+
 	return cdroms_list;
 }
 #endif /* __linux__ */
@@ -809,15 +889,16 @@ cd_drive_free (CDDrive *drive)
 	g_free (drive->display_name);
 	g_free (drive->cdrecord_id);
 	g_free (drive->device);
+	g_free (drive);
 }
 
-CDBlankMediaType
+CDMediaType
 guess_media_type (const char *device_path)
 {
 #if defined(__linux__) || defined(__FreeBSD__)
 	return linux_bsd_media_type (device_path);
 #else
-	return CD_BLANK_MEDIA_TYPE_UNKNOWN;
+	return CD_MEDIA_TYPE_UNKNOWN;
 #endif
 }
 
