@@ -52,6 +52,8 @@
 #include "mkdtemp.h"
 #endif
 
+extern gboolean is_audio (char *mime_type);
+
 static void
 write_all (int fd, char *buf, int len)
 {
@@ -467,6 +469,9 @@ cancel_mkisofs (gpointer data)
 	g_main_loop_quit (mkisofs_output->loop);
 }
 
+/**
+ * Create an ISO image in filename from the data files in burn:///
+ */
 int
 make_iso (const char *filename, const char *label, gboolean warn_low_space)
 {
@@ -680,41 +685,43 @@ make_iso (const char *filename, const char *label, gboolean warn_low_space)
 	return mkisofs_output.result;
 }
 
-gint 
-build_list_of_music_arguments (char *uri, int *argc, const char **argv, int max_num_args)
+/**
+ * Build a GList* of Track pointers from the audio tracks in burn:///
+ */
+GList *
+build_list_of_audio_tracks (void)
 {
+	GList *list;
 	GnomeVFSFileInfo *info;
 	GnomeVFSDirectoryHandle *handle;
 	GnomeVFSResult result;
 	char *backing;
-	gint count;
-
-	count = 0;
 	
-	result = gnome_vfs_directory_open (&handle, uri, GNOME_VFS_FILE_INFO_FOLLOW_LINKS | GNOME_VFS_FILE_INFO_GET_MIME_TYPE);
+	result = gnome_vfs_directory_open (&handle, "burn:///", GNOME_VFS_FILE_INFO_FOLLOW_LINKS | GNOME_VFS_FILE_INFO_GET_MIME_TYPE);
 	if (result != GNOME_VFS_OK) {
 		return 0;
 	}
 	
 	info = gnome_vfs_file_info_new ();
 
+	list = NULL;
 	while (TRUE) {
 		result = gnome_vfs_directory_read_next (handle, info);
 		if (result != GNOME_VFS_OK)
 			break;
 
 		if (is_audio (info->mime_type)) {
+			Track *track;
 			backing = get_backing_file (info->name);
-			argv[(*argc)++] = backing;
-			count++;
-		}
-		
-		if (count == max_num_args) {
-			break;		
+			track = g_new0 (Track, 1);
+			track->type = TRACK_TYPE_AUDIO;
+			track->audio.filename = backing;
+			track->audio.cdtext = NULL;
+			list = g_list_prepend (list, track);
 		}
 	}
 
 	gnome_vfs_directory_close (handle);
 	gnome_vfs_file_info_unref (info);
-	return count;
+	return g_list_reverse (list);
 }
