@@ -23,5 +23,4 @@
 */
 
 int make_iso (const char *filename, const char *label, gboolean warn_low_space);
-gint build_list_of_music_arguments (char *uri, int *argc, const char **argv, int max_num_args);
-extern gboolean is_audio (char *mime_type);
+GList *build_list_of_audio_tracks (void);
